<?php

namespace Drupal\hubspot_forms\Form;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Config\TypedConfigManagerInterface;

/**
 * Configure site information settings for this site.
 */
class HubspotSettingsForm extends ConfigFormBase {

  /**
   * The cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a QuicklinkConfigForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config_manager
   *   The typed config manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   * @param DateFormatterInterface $date_formatter
   */
  public function __construct(ConfigFactoryInterface $config_factory, TypedConfigManagerInterface $typed_config_manager, CacheBackendInterface $cache, DateFormatterInterface $date_formatter) {
    parent::__construct($config_factory, $typed_config_manager);
    $this->cache = $cache;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('cache.default'),
      $container->get('date.formatter')
    );
  }


  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'hubspot_forms_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['hubspot_forms.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('hubspot_forms.settings');

    $form['hubspot_access_type'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Hubspot Access Type'),
      '#default_value' => $config->get('hubspot_access_type'),
      '#options'   => [
        0 => $this->t('API Key (Legacy)'),
        1 => $this->t('Access Token'),
      ],
    ];

    $form['hubspot_api_key'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Hubspot API Key (Legacy method)'),
      '#default_value' => $config->get('hubspot_api_key'),
      '#description'   => $this->t('Please use <strong>demo</strong> to load example forms. Generate a @newkey. Make sure to clear Drupal cache after you change API Key. <strong>Warning</strong>: @keysunset.', [
        '@newkey' => Link::fromTextAndUrl($this->t('new key'), Url::fromUri('https://app.hubspot.com/keys/get'))->toString(),
        '@keysunset' => Link::fromTextAndUrl($this->t('API Key Sunset'), Url::fromUri('https://developers.hubspot.com/changelog/upcoming-api-key-sunset'))->toString(),
      ]),
      '#states' => [
        'visible' => [':input[name="hubspot_access_type"]' => ['value' => 0]],
        'required' => [':input[name="hubspot_access_type"]' => ['value' => 0]],
      ],
    ];

    $form['hubspot_access_token'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Hubspot Access Token'),
      '#default_value' => $config->get('hubspot_access_token'),
      '#description'   => $this->t('Provide Private App access token. Refer to the @privateappsdocs.', [
        '@privateappsdocs' => Link::fromTextAndUrl($this->t('Private Apps documentation'), Url::fromUri('https://developers.hubspot.com/docs/api/private-apps'))->toString(),
      ]),
      '#states' => [
        'visible' => [':input[name="hubspot_access_type"]' => ['value' => 1]],
        'required' => [':input[name="hubspot_access_type"]' => ['value' => 1]],
      ],
    ];

    $form['hubspot_portal_id'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Hubspot Portal ID'),
      '#default_value' => $config->get('hubspot_portal_id'),
      '#description'   => $this->t('In your HubSpot account, navigate to Dashboard'),
      '#states' => [
        'visible' => [':input[name="hubspot_access_type"]' => ['value' => 1]],
        'required' => [':input[name="hubspot_access_type"]' => ['value' => 1]],
      ],
    ];

    $period = [0, 60, 180, 300, 600, 900, 1800, 2700, 3600, 10800, 21600, 32400, 43200, 86400];
    $period = array_map([$this->dateFormatter, 'formatInterval'], array_combine($period, $period));
    $period[0] = '<' . $this->t('no caching') . '>';
    $form['caching'] = [
      '#type' => 'select',
      '#title' => $this->t('Hubspot forms caching'),
      '#options' => $period,
      '#description' => $this->t('This is used as the value for Hubspot forms caching time. Caching is recommended for better performance.'),
      '#default_value' => ($caching = $config->get('caching')) ? $caching : 10800, // defaults to 3 hours.
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('hubspot_forms.settings');
    $config
      ->set('hubspot_access_type', $form_state->getValue('hubspot_access_type'))
      ->set('hubspot_api_key', $form_state->getValue('hubspot_api_key'))
      ->set('hubspot_access_token', $form_state->getValue('hubspot_access_token'))
      ->set('hubspot_portal_id', $form_state->getValue('hubspot_portal_id'))
      ->set('caching', $form_state->getValue('caching'))
      ->save();
    $this->cache->delete('hubspot_forms');
    parent::submitForm($form, $form_state);
  }

}
