<?php

namespace Drupal\hubspot_forms;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Exception\ClientException;

/**
 * Hubspot forms service class.
 *
 * @package hubspot_forms
 */
class HubspotForms {

  use StringTranslationTrait;

  const API_ENDPOINT = 'https://api.hubapi.com/';

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * An http client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Logger Factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory, ClientInterface $http_client, CacheBackendInterface $cache, LoggerChannelFactoryInterface $loggerFactory) {
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
    $this->cache = $cache;
    $this->loggerFactory = $loggerFactory->get('hubspot_forms');
  }

  /**
   * Get form ids.
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function getFormIds() {
    $config = $this->configFactory->get('hubspot_forms.settings');
    $expire = ($val = $config->get('caching')) ? $val : 10800; // defaults to 3 hours.
    if (empty($expire)) {
      $forms = $this->fetchHubspotForms();
    }
    else {
      if ($cache = $this->cache->get('hubspot_forms')) {
        $forms = $cache->data;
      }
      else {
        $forms = $this->fetchHubspotForms();
        $this->cache->set('hubspot_forms', $forms, $expire);
      }
    }
    $form_ids = ['' => $this->t('Choose a Hubspot form')];
    if (!empty($forms)) {
      $form_ids = array_merge($form_ids, $forms);
    }
    return $form_ids;
  }

  /**
   * Make call to Hubspot Forms API and get a list of all available forms.
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function fetchHubspotForms() {
    $forms = [];
    $config = $this->configFactory->get('hubspot_forms.settings');
    $access_type = trim($config->get('hubspot_access_type'));

    if (!empty($access_type)) {
      $access_token = $config->get('hubspot_access_token');
      $fetchedForms = $this->fetchFormsByAccessToken($access_token);
      usort($fetchedForms, function ($a, $b) {
        return strtotime($b->createdAt) - strtotime($a->createdAt);
      });
      $portal_id = trim($config->get('hubspot_portal_id'));
      foreach ($fetchedForms as $form) {
        $forms[$portal_id . '::' . $form->id] = $form->name;
      }
    }
    else {
      // Use Legacy method.
      $api_key = $config->get('hubspot_api_key');
      $fetchedForms = $this->fetchFormsByApiKey($api_key);
      usort($fetchedForms, function ($a, $b) {
        return $b->createdAt - $a->createdAt;
      });
      if (!empty($fetchedForms)) {
        foreach ($fetchedForms as $item) {
          $forms[$item->portalId . '::' . $item->guid] = $item->name;
        }
      }
    }

    return $forms;
  }

  /**
   * Fetch forms using Access Token.
   *
   * @param string $access_token
   *   Access Token.
   *
   * @return array
   *   Array of Forms
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  private function fetchFormsByAccessToken($access_token) {
    // Get all forms from a portal.
    // See https://developers.hubspot.com/docs/api/marketing/forms
    $options = ['limit' => 100];
    $more_forms = false;
    $results = [];
    do {
      try {
        $request = $this->httpClient->request('GET', self::API_ENDPOINT . 'marketing/v3/forms/', [
          'query' => $options,
          'timeout' => 5,
          'headers' => [
            'Accept' => 'application/json',
            'authorization' => "Bearer $access_token",
          ],
        ]);

        if ($request->getStatusCode() == 200) {
          $response = json_decode($request->getBody());
          if (!empty($response) && !empty($response->results)) {
            $results = array_merge($results, $response->results);
          }
          if (property_exists($response, 'paging')) {
            $options['after'] = $response->paging->next->after;
            $more_forms = TRUE;
          }
          else {
            $more_forms = FALSE;
          }
        }
      }
      catch (ClientException $e) {
        $this->loggerFactory->notice($e->getMessage() . '. Make sure you provided correct Hubspot Access Token on the configuration page.');
        break;
      }
      catch (ConnectException $e) {
        $this->loggerFactory->notice($e->getMessage());
        break;
      }
    } while ($more_forms);
    return $results;
  }

  /**
   * Fetch forms using API KEY.
   *
   * @param string $api_key
   *   API Key.
   *
   * @return array
   *   Array of Forms
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  private function fetchFormsByApiKey($api_key) {
    try {
      // Get all forms from a portal.
      // See http://developers.hubspot.com/docs/methods/forms/v2/get_forms
      $request = $this->httpClient->request('GET', self::API_ENDPOINT . 'forms/v2/forms?hapikey=' . $api_key, [
        'timeout' => 5,
        'headers' => [
          'Accept' => 'application/json'
        ]
      ]);
      if ($request->getStatusCode() == 200) {
        $response = json_decode($request->getBody());
        if (!empty($response)) {
          return $response;
        }
      }
      return [];
    }
    catch (ClientException $e) {
      $this->loggerFactory->notice($e->getMessage() . '. Make sure correct Hubspot API Key provided.');
      return [];
    }
    catch (ConnectException $e) {
      $this->loggerFactory->notice($e->getMessage());
      return [];
    }
  }

  /**
   * Check Hubspot connection.
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function isConnected() {
    $forms = $this->fetchHubspotForms();
    return count($forms);
  }

}
